Option Explicit On
Option Strict On
Option Compare Binary

Imports System.IO

''' <remarks>
''' Eine einfache Klasse zum rekursiven
''' Enumerieren von Dateien und bzw. oder
''' Ordnern in einem Dateisystem.
''' </remarks>
Public Class FileSystemEnumerator
    Private m_StartDirectory As DirectoryInfo
    Private m_EnumerationStatus As EnumerationType = EnumerationType.Both

    Public Enum EnumerationType As Integer
        OnlyDirs = 1
        OnlyFiles = 2
        Both = 0
    End Enum

    ''' <summary>
    ''' Wird ausgelst, wenn eine Datei gefunden wird.
    ''' </summary>
    ''' <param name="File">Gefundene Datei.</param>
    Public Event FileFound(ByVal File As IO.FileInfo)

    ''' <summary>
    ''' Wird ausgelst, wenn ein Ordner gefunden wird.
    ''' </summary>
    ''' <param name="Directory">Gefundener Ordner.</param>
    ''' <param name="ContinueRecursion">
    ''' Boolescher Parameter, in dem angebenen wird,
    ''' ob die rekursive Enumeration fortgesetzt
    ''' werden soll.
    ''' </param>
    Public Event DirectoryFound(ByVal Directory As IO.DirectoryInfo, _
        ByRef ContinueRecursion As Boolean)

    ''' <summary>
    ''' Wird ausgelst, wenn der Zufriff auf das Dateisystem
    ''' verweigert wurde und ein andere Fehler auftrat.
    ''' </summary>
    ''' <param name="ex">Fehler</param>
    Public Event ErrorOccoured(ByVal ex As Exception)

    ''' <summary>
    ''' Gibt den Ordner, in dem die Enumeration
    ''' gestartet werden soll, an oder gibt ihn
    ''' zurck.
    ''' </summary>
    ''' <value>Ordner, in dem die Enumeration
    ''' gestartet werden soll.</value>
    Public Property StartDirectory() As DirectoryInfo
        Get
            Return m_StartDirectory
        End Get
        Set(ByVal Value As DirectoryInfo)
            If Value.Exists Then
                m_StartDirectory = Value
            Else
                Throw New DirectoryNotFoundException
            End If
        End Set
    End Property

    ''' <summary>
    ''' Gibt den Enueration-Status an oder gibt ihn
    ''' zurck.
    ''' </summary>
    ''' <value>Status mit dem die Enumeration
    ''' arbeitet.</value>
    Public Property EnumerationStatus() As EnumerationType
        Get
            Return m_EnumerationStatus
        End Get
        Set(ByVal value As EnumerationType)
            m_EnumerationStatus = value
        End Set
    End Property

    ''' <summary>
    ''' Startet die Enumeration der Objekte in
    ''' der Eigenschaft <c>StartDirectory</c>
    ''' angegebenen Ordner.
    ''' </summary>
    Public Sub Enumerate()
        If m_StartDirectory.Exists Then
            CheckDirectory(m_StartDirectory)
        Else
            Throw New DirectoryNotFoundException
        End If
    End Sub

    ''' <summary>
    ''' Startet die Enumeration aller Dateien, die
    ''' direkt im Ordner liegen und geht dann
    ''' alle Ordner, die direkt in einem Ordner
    ''' liegen, durch und lst fr jeden Ordner
    ''' das Ereignis DirectoryFound aus. Dann wird
    ''' ggf. die Enumeration rekursiv auf enthaltene
    ''' Ordner fortgesetzt.
    ''' </summary>
    ''' <param name="Dir">Ordner, dessen
    ''' Inhalt enumeriert werden soll.</param>
    Private Sub CheckDirectory(ByVal Dir As IO.DirectoryInfo)
        If m_EnumerationStatus <> EnumerationType.OnlyDirs Then
            CheckFiles(Dir)
        End If

        Dim di As IO.DirectoryInfo
        Try
            For Each di In Dir.GetDirectories
                Dim b As Boolean = True
                If m_EnumerationStatus <> EnumerationType.OnlyFiles Then
                    RaiseEvent DirectoryFound(di, b)
                End If

                If b Then
                    CheckDirectory(di)
                End If
            Next di
        Catch ex As Exception
            RaiseEvent ErrorOccoured(ex)
        End Try
    End Sub

    ''' <summary>
    ''' Geht alle Dateien, die direkt in
    ''' einem Ordner liegen, durch und lst fr jede
    ''' Datei das Ereignis FileFound aus.
    ''' </summary>
    ''' <param name="Dir">Ordner, dessen Dateien
    ''' enumeriert werden sollen.</param>
    Private Sub CheckFiles(ByVal Dir As IO.DirectoryInfo)
        Dim fi As IO.FileInfo
        Try
            For Each fi In Dir.GetFiles
                RaiseEvent FileFound(fi)
            Next fi
        Catch ex As Exception
            RaiseEvent ErrorOccoured(ex)
        End Try
    End Sub
End Class